<?php
/* --------------------------------------------------------------
   ShopOfflineApplicationTopExtender.inc.php 2015-02-23 gm
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2015 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

class ShopOfflineApplicationTopExtender extends ShopOfflineApplicationTopExtender_parent
{
	public function proceed()
	{
		parent::proceed();
		
		if($this->_shopIsOffline() && !$this->_isAdmin() && !$this->_attemptLogin())
		{
			// lightweight and manual controller execution
			$httpContextReader     = MainFactory::create(HttpContextReader::class);
			$httpResponseProcessor = MainFactory::create(HttpResponseProcessor::class);
			$defaultContentView    = MainFactory::create(ContentView::class);
			
			/** @var \ShopOfflineController $controller */
			$controller = MainFactory::create(ShopOfflineController::class, $httpContextReader, $httpResponseProcessor,
			                                  $defaultContentView);
			
			$httpResponseProcessor->proceed($controller->actionDefault());
			exit;
		}
		elseif(!$this->_releaseInfoMatchInstalledVersion())
		{
			// do not process ShopOfflineController here, because the shop will redirect the user to the gambio_updater
			define('SHOP_OFFLINE', true);
		}
		else
		{
			define('SHOP_OFFLINE', false);
		}
	}
	
	
	/**
	 * Checks if the shop software is set to offline mode.
	 *
	 * @return bool True if application is in offline mode.
	 */
	protected function _shopIsOffline()
	{
		return gm_get_conf('GM_SHOP_OFFLINE') === 'checked';
	}
	
	
	/**
	 * Checks if the current logged in user is an admin.
	 *
	 * @return bool True if the current user is an admin.
	 */
	protected function _isAdmin()
	{
		return $_SESSION['customers_status']['customers_status_id'] === '0';
	}
	
	
	/**
	 * Checks if release info match the installed version.
	 *
	 * @return bool True if release info match the installed version.
	 */
	protected function _releaseInfoMatchInstalledVersion()
	{
		include(DIR_FS_CATALOG . 'release_info.php');
		$installedVersion = $gx_version;
		
		// do not use gm_get_conf() to avoid caching problems
		$query  = 'SELECT `gm_value` FROM `gm_configuration` WHERE `gm_key` = "INSTALLED_VERSION" LIMIT 1';
		$result = xtc_db_query($query);
		if(xtc_db_num_rows($result) == 1)
		{
			$row              = xtc_db_fetch_array($result);
			$installedVersion = $row['gm_value'];
		}
		
		return $gx_version === $installedVersion;
	}
	
	
	/**
	 * Checks if login is attempted.
	 *
	 * @return bool True if login is attempted.
	 */
	protected function _attemptLogin()
	{
		return (count($_POST) > 0 && array_key_exists('email_address', $_POST) && array_key_exists('password', $_POST));
	}
}